/**
 * RTD App and Service 
 * Purpose:	Automatic Measurement of Random Mobile App Performance
 * Built on Eclipse for Linux, Ubuntu 12.04 LTS
 * By:	Ramon Luis De Jesus
 * On:	Jun 15, 2013	2:37pm
 * Note:	Uses RootTools-2.6.jar from Stericson (https://code.google.com/p/roottools/)
 */
package gov.nist.rtd;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.os.BadParcelableException;
import android.os.Bundle;
import android.os.Environment;
import android.os.Vibrator;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Menu;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends Activity {

	/** Global Values */
	Vibrator vibrate = null;

	// Controls
	CheckBox chbAutoTrace;
	CheckBox chbAutoKill;

	EditText etSeconds;
	EditText etTimes;
	EditText etTraceName;

	static ProgressBar pbProgress;

	// List of apps
	private ListView lstAppsContext;
	private ArrayAdapter<String> myarrayAdapter;
	private List<String> myApps;
	private List<ApplicationInfo> packages;

	// To display dates on notification
	public static String DateStarted;
	public static String DateFinished;

	// Use to store and later delete our traceFiles
	public static File TracesDirectory;

	// Access anywhere
	public static String Packagename = ""; // Stores and displays the app we are
											// tracing.
	public static String TraceName = "trace"; // Default trace name it can
												// change.
	public static int Seconds = 1; // Default seconds for buffer and rename
									// process.
	public static int Times = 10; // Default times to trace app.

	public static boolean AutoTrace = false; // ~Obsolete now. Great idea, but
												// doesn't work if we can't
												// actually trace a deployed
												// app.
	public static boolean AutoKill = false; // Good, if root access is available
											// and user wants to kill the
											// selected app once its not
											// running.

	// List stores elapsed service side times of our running apps.
	public static List<String> ElapsedTimes;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		WhoAmI();

		TracesDirectory = new File(Environment.getExternalStorageDirectory()
				+ "/Traces");
		TracesDirectory.mkdirs();

		vibrate = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);

		pbProgress = (ProgressBar) findViewById(R.id.pbProgress);
		pbProgress.setVisibility(View.GONE);

		// Keep screen on, even if the device its not charging.
		// Bug fixer for stops in the service.
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);	

		/** Listeners */
		lstAppsContext = (ListView) findViewById(R.id.lstApps);
		lstAppsContext.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int arg2,
					long arg3) {
				btnStopClicked(null);
				Packagename = (GetInstalledAppsOrGetInfo(((TextView) arg1)
						.getText()) != null) ? GetInstalledAppsOrGetInfo(((TextView) arg1)
						.getText()).packageName.toString() : "error";
				// new String(packages.get(arg2).packageName);
				Log.d("Package is:", Packagename);
				// Vibrate twice to let you know what you selected...
				vibrate.vibrate(50);
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				vibrate.vibrate(50);
				if (Packagename != null) {
					MessageBox("App ready for trace.");
				} else {
					MessageBox("Couldn't find app, or its not accesible!");
				}
			}

		});

		etSeconds = (EditText) findViewById(R.id.etSeconds);
		etSeconds.addTextChangedListener(new TextWatcher() {
			@Override
			public void afterTextChanged(Editable s) {
				try {
					if (etSeconds.getText().toString().length() > 0) {
						int x = Integer
								.parseInt(etSeconds.getText().toString());
						if (x > 0 && x < 2000000000) {
							Seconds = x;
						} else {
							throw new BadParcelableException("");
						}
					} else {
						Seconds = Integer.parseInt(etSeconds.getHint()
								.toString());
					}
				} catch (Exception ex) {
					MessageBox("Invalid number of buffer to trace!");
				}
			}

			@Override
			public void beforeTextChanged(CharSequence s, int start, int count,
					int after) {
			}

			@Override
			public void onTextChanged(CharSequence s, int start, int before,
					int count) {
			}
		});

		etTimes = (EditText) findViewById(R.id.etTimes);
		etTimes.addTextChangedListener(new TextWatcher() {
			@Override
			public void afterTextChanged(Editable s) {
				try {
					if (etTimes.getText().toString().length() > 0) {
						int x = Integer.parseInt(etTimes.getText().toString());
						if (x > 0 && x < 2000000000) {
							Times = x;
						} else {
							throw new BadParcelableException("");
						}
					} else {
						Times = Integer.parseInt(etTimes.getHint().toString());
					}
				} catch (Exception ex) {
					MessageBox("Invalid number of buffer to trace!");
				}
			}

			@Override
			public void beforeTextChanged(CharSequence s, int start, int count,
					int after) {
			}

			@Override
			public void onTextChanged(CharSequence s, int start, int before,
					int count) {
			}
		});

		etTraceName = (EditText) findViewById(R.id.etTraceName);
		etTraceName.addTextChangedListener(new TextWatcher() {
			@Override
			public void afterTextChanged(Editable s) {
				try {
					if (etTraceName.getText().toString().length() > 0) {
						String x = etTraceName.getText().toString();
						if (x.length() > 0) {
							TraceName = x;
						} else {
							throw new BadParcelableException("");
						}
					} else {
						TraceName = etTraceName.getHint().toString();
					}
				} catch (Exception ex) {
					MessageBox("Invalid number of buffer to trace!");
				}
			}

			@Override
			public void beforeTextChanged(CharSequence s, int start, int count,
					int after) {
			}

			@Override
			public void onTextChanged(CharSequence s, int start, int before,
					int count) {
			}
		});

		chbAutoTrace = (CheckBox) findViewById(R.id.chbAutoTrace);
		chbAutoTrace.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(CompoundButton buttonView,
					boolean isChecked) {
				AutoTrace = isChecked;
				vibrate.vibrate(50);
			}
		});

		chbAutoKill = (CheckBox) findViewById(R.id.chbAutoKill);
		chbAutoKill.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			@Override
			public void onCheckedChanged(CompoundButton buttonView,
					boolean isChecked) {
				AutoKill = isChecked;
				vibrate.vibrate(50);
			}
		});

		/** Methods to be run at start */
		GetInstalledAppsOrGetInfo(null);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	public void onDestroy() {
		super.onDestroy();
		System.exit(0);
	}

	/**
	 * Global Click Events
	 */
	public void btnStartClicked(View v) {
		if (Packagename != null && Packagename != "" && Packagename != "error") {
			vibrate.vibrate(50);

			// Clean old trace files with same name to skip runtime errors.
			btnCleanClicked(null);

			// Start the service.
			startService(new Intent(getBaseContext(), AppLauncher.class));
		}

		// ~ Optional close the service window; Un-comment the below line.
		// finish();
	}

	public void btnStopClicked(View v) {
		vibrate.vibrate(50);
		// Stop the service on the service class
		stopService(new Intent(getBaseContext(), AppLauncher.class));
	}

	public void btnCleanClicked(View v) {
		vibrate.vibrate(50);

		pbProgress.setVisibility(View.VISIBLE);
		DeleteTraceFiles delete = new DeleteTraceFiles();
		delete.Start();

	}

	public void btnRefreshListClicked(View v) {

		GetInstalledAppsOrGetInfo(null);
		vibrate.vibrate(50);
	}

	public void btnExitClicked(View v) {

		btnStopClicked(null);
		vibrate.vibrate(50);
		finish();
	}

	/**
	 * Misc
	 */
	public void MessageBox(String message) {
		Toast.makeText(this, message, Toast.LENGTH_LONG).show();
	}

	public static void DeleteCompleted() {
		pbProgress.setVisibility(View.GONE);
	}

	// Gets or displays Installed apps or Get Info
	ApplicationInfo GetInstalledAppsOrGetInfo(CharSequence getWithName) {
		// get the package manager
		final PackageManager pm = getPackageManager();
		// List all installed packages
		packages = pm.getInstalledApplications(PackageManager.GET_META_DATA);

		// Assign packages to the list here
		if (packages.size() > 0) {
			if (!(getWithName != null && getWithName != "")) {
				myApps = new ArrayList<String>();

				for (int i = 0; i < packages.size(); i++) {
					try {
						if (packages.get(i).packageName.contains("val")) {
							myApps.add(new String(packages.get(i).loadLabel(pm)
									.toString()));
						}
					} catch (Exception e) {
						MessageBox(e.getLocalizedMessage());
						e.printStackTrace();
					}
				}

				if (myApps != null && myApps.size() > 0) {
					try {
						myarrayAdapter = new ArrayAdapter<String>(this,
								android.R.layout.simple_list_item_1, myApps);
						lstAppsContext.setAdapter(myarrayAdapter);
						lstAppsContext.setTextFilterEnabled(true);
						myarrayAdapter.notifyDataSetChanged();
					} catch (Exception e) {
						MessageBox(e.getMessage());
						e.printStackTrace();
					}
				}
			} else {
				for (int i = 0; i < packages.size(); i++) {
					try {
						if (new String(packages.get(i).loadLabel(pm).toString())
								.equals(getWithName.toString())) {
							return packages.get(i);
						}
					} catch (Exception e) {
						MessageBox(e.getLocalizedMessage());
						e.printStackTrace();
					}
				}
			}
		}
		return null;
	}

	void WhoAmI() {
		Log.e("~", "************************");
		Log.d("*", "RTD App and Service");
		Log.d("*",
				"Purpose:	Automatic Measurement of Random Mobile App Performance");
		Log.d("*", "Built on Eclipse for Linux, Ubuntu 12.04 LTS");
		Log.d("*", "By:	Ramon Luis De Jesus");
		Log.d("*", "On:	Jun 15, 2013	2:37pm");
		Log.d("*",
				"Note:	Uses RootTools-2.6.jar from Stericson (https://code.google.com/p/roottools/)");
		Log.e("~", "************************");
	}
}
